@@ -108,14 +108,15 @@ const char *short_help =
     "\n"
     "MP4 specific options:\n"
     "  -w\t\tWrap AAC data in MP4 container. (default for *.mp4 and *.m4a)\n"
+    "  --tag <tagname,tagvalue> Add named tag (iTunes '----')\n"
     "  --artist X\tSet artist to X\n"
     "  --composer X\tSet composer to X\n"
     "  --title X\tSet title to X\n"
-    "  --genre X\tSet genre to X\n"
+    "  --genre X\tSet genre number to X\n"
     "  --album X\tSet album to X\n"
     "  --compilation\tSet compilation\n"
-    "  --track X\tSet track number to X\n"
-    "  --disc X\tSet disc number to X\n"
+    "  --track X\tSet track to X (number/total)\n"
+    "  --disc X\tSet disc to X (number/total)\n"
     "  --year X\tSet year to X\n"
     "  --cover-art X\tRead cover art from file X\n"
     "  --comment X\tSet comment to X\n"
@@ -174,11 +175,11 @@ const char *long_help =
     "  --artist X\tSet artist to X\n"
     "  --composer X\tSet composer to X\n"
     "  --title X\tSet title/track name to X\n"
-    "  --genre X\tSet genre to X\n"
+    "  --genre X\tSet genre number to X\n"
     "  --album X\tSet album/performer to X\n"
     "  --compilation\tMark as compilation\n"
-    "  --track X\tSet track number to X\n"
-    "  --disc X\tSet disc number to X\n"
+    "  --track X\tSet track to X (number/total)\n"
+    "  --disc X\tSet disc to X (number/total)\n"
     "  --year X\tSet year to X\n"
     "  --cover-art X\tRead cover art from file X\n"
     "\t\tSupported image formats are GIF, JPEG, and PNG.\n"
@@ -309,6 +310,7 @@ enum flags
     COVER_ART_FLAG,
     COMMENT_FLAG,
     WRITER_FLAG,
+    TAG_FLAG,
 };
 
 #ifndef _WIN32
@@ -423,11 +425,12 @@ int main(int argc, char *argv[])
 
     FILE *outfile = NULL;
 
-    unsigned int trackno = 0;
-    unsigned int discno = 0;
+    unsigned int ntracks = 0, trackno = 0;
+    unsigned int ndiscs = 0, discno = 0;
     uint8_t compilation = 0;
     const char *artist = NULL, *title = NULL, *album = NULL, *year = NULL,
-        *genre = NULL, *comment = NULL, *composer = NULL;
+        *comment = NULL, *composer = NULL, *tagname = 0, *tagval = 0;
+    int genre = 0;
     uint8_t *artData = NULL;
     uint64_t artSize = 0;
     uint64_t encoded_samples = 0;
@@ -480,6 +483,7 @@ int main(int argc, char *argv[])
             {"compilation", 0, 0, COMPILATION_FLAG},
             {"pcmswapbytes", 0, 0, 'X'},
             {"ignorelength", 0, 0, IGNORELEN_FLAG},
+            {"tag", 1, 0, TAG_FLAG},
             {0, 0, 0, 0}
         };
         int c = -1;
@@ -599,16 +603,21 @@ int main(int argc, char *argv[])
             album = optarg;
             break;
         case TRACK_FLAG:
-            trackno = atoi(optarg);
+            if (sscanf(optarg, "%d/%d", &trackno, &ntracks) < 1)
+                dieMessage = "Wrong track number.\n";
             break;
         case DISC_FLAG:
-            discno = atoi(optarg);
+            if (sscanf(optarg, "%d/%d", &discno, &ndiscs) < 1)
+                dieMessage = "Wrong disc number.\n";
             break;
         case COMPILATION_FLAG:
             compilation = 0x1;
             break;
         case GENRE_FLAG:
-            genre = optarg;
+            genre = atoi(optarg);
+            if ((genre < 0) || (genre > 146))
+                dieMessage = "Genre number out of range.\n";
+            genre++;
             break;
         case YEAR_FLAG:
             year = optarg;
@@ -616,6 +625,14 @@ int main(int argc, char *argv[])
         case COMMENT_FLAG:
             comment = optarg;
             break;
+        case TAG_FLAG:
+            tagname = optarg;
+            if (!(tagval = strchr(optarg, ',')))
+                dieMessage = "Missing tag value.\n";
+            else
+                *(char *)tagval++ = 0;
+            mp4tag_add(tagname, tagval);
+            break;
         case COVER_ART_FLAG:
             {
                 FILE *artFile = fopen(optarg, "rb");
@@ -810,9 +827,9 @@ int main(int argc, char *argv[])
         return 1;
     }
 
-    if (container != MP4_CONTAINER && (trackno || artist ||
+    if (container != MP4_CONTAINER && (ntracks || trackno || artist ||
                                        title || album || year || artData ||
-                                       genre || comment || discno ||
+                                       genre || comment || discno || ndiscs ||
                                        composer || compilation))
     {
         fprintf(stderr, "Metadata requires MP4 output!\n");
@@ -1130,7 +1147,9 @@ int main(int argc, char *argv[])
         SETTAG(title);
         SETTAG(album);
         SETTAG(trackno);
+        SETTAG(ntracks);
         SETTAG(discno);
+        SETTAG(ndiscs);
         SETTAG(compilation);
         SETTAG(year);
         SETTAG(genre);
