@@ -103,11 +103,12 @@ static int ftypout(void)
 {
     int size = 0;
 
-    size += stringout("isom");
+    size += stringout("M4A ");
     size += u32out(0);
     size += stringout("M4A ");
     size += stringout("mp42");
-    size += stringout("mp41");
+    size += stringout("isom");
+    size += u32out(0);
 
     return size;
 }
@@ -527,9 +528,9 @@ static int tagtxt(char *tagname, const char *tagtxt)
     return size;
 }
 
-static int tagu32(char *tagname, int n /*number of stored fields*/)
+static int tagu16(char *tagname, int n /*number of stored fields*/)
 {
-    int numsize = n * 4;
+    int numsize = n * 2;
     int size = 0;
     int datasize = numsize + 16;
 
@@ -537,7 +538,23 @@ static int tagu32(char *tagname, int n /*number of stored fields*/)
     size += dataout(tagname, 4);
     size += u32out(datasize);
     size += dataout("data", 4);
-    size += u32out(0); // data type uint32
+    size += u32out(0); // data type uint16
+    size += u32out(0);
+
+    return size;
+}
+
+static int tagu8(char *tagname, int n /*number of stored fields*/)
+{
+    int numsize = n * 1;
+    int size = 0;
+    int datasize = numsize + 16;
+
+    size += u32out(datasize + 8);
+    size += dataout(tagname, 4);
+    size += u32out(datasize);
+    size += dataout("data", 4);
+    size += u32out(0x15); // data type uint8
     size += u32out(0);
 
     return size;
@@ -592,6 +609,7 @@ static int hdlr2out(void)
 static int ilstout(void)
 {
     int size = 0;
+    int cnt;
 
     size += tagtxt("\xa9" "too", mp4config.tag.encoder);
     if (mp4config.tag.artist)
@@ -601,25 +619,32 @@ static int ilstout(void)
     if (mp4config.tag.title)
         size += tagtxt("\xa9" "nam", mp4config.tag.title);
     if (mp4config.tag.genre)
-        size += tagtxt("gnre", mp4config.tag.genre);
+    {
+        size += tagu16("gnre", 1);
+        size += u16out(mp4config.tag.genre);
+    }
     if (mp4config.tag.album)
         size += tagtxt("\xa9" "alb", mp4config.tag.album);
     if (mp4config.tag.compilation)
     {
-        size += tagu32("cpil", 1);
-        size += u32out(mp4config.tag.compilation);
+        size += tagu8("cpil", 1);
+        size += u8out(mp4config.tag.compilation);
     }
     if (mp4config.tag.trackno)
     {
-        size += tagu32("trkn", 2);
-        size += u32out(mp4config.tag.trackno);
-        size += u32out(0);
+        size += tagu16("trkn", 4);
+        size += u16out(0);
+        size += u16out(mp4config.tag.trackno);
+        size += u16out(mp4config.tag.ntracks);
+        size += u16out(0);
     }
     if (mp4config.tag.discno)
     {
-        size += tagu32("disk", 2);
-        size += u32out(mp4config.tag.discno);
-        size += u32out(0);
+        size += tagu16("disk", 4);
+        size += u16out(0);
+        size += u16out(mp4config.tag.discno);
+        size += u16out(mp4config.tag.ndiscs);
+        size += u16out(0);
     }
     if (mp4config.tag.year)
         size += tagtxt("\xa9" "day", mp4config.tag.year);
@@ -631,6 +656,33 @@ static int ilstout(void)
     if (mp4config.tag.comment)
         size += tagtxt("\xa9" "cmt", mp4config.tag.comment);
 
+    // ----(mean(com.apple.iTunes),name(name),data(data))
+    for (cnt = 0; cnt < mp4config.tag.extnum; cnt++)
+    {
+        static const char *mean = "faac";//"com.apple.iTunes";
+        const char *name = mp4config.tag.ext[cnt].name;
+        const char *data = mp4config.tag.ext[cnt].data;
+        uint32_t len1 = 8 + strlen(mean) + 4;
+        uint32_t len2 = 8 + strlen(name) + 4;
+        uint32_t len3 = 8 + strlen(data) + 4 + 4;
+        u32out(8 + len1 + len2 + len3);
+        size += 8 + len1 + len2 + len3;
+        stringout("----");
+        u32out(len1);
+        stringout("mean");
+        u32out(0);
+        stringout(mean);
+        u32out(len2);
+        stringout("name");
+        u32out(0);
+        stringout(name);
+        u32out(len3);
+        stringout("data");
+        u32out(1);
+        u32out(0);
+        stringout(data);
+    }
+
     return size;
 };
 
@@ -825,3 +877,20 @@ int mp4atom_tail(void)
 
     return 0;
 }
+
+int mp4tag_add(const char *name, const char *data)
+{
+    int idx = mp4config.tag.extnum;
+
+    if (idx >= TAGMAX)
+    {
+        fprintf(stderr, "To many tags\n");
+        return -1;
+    }
+
+    mp4config.tag.ext[idx].name = name;
+    mp4config.tag.ext[idx].data = data;
+    mp4config.tag.extnum++;
+
+    return 0;
+}
