@@ -16,7 +16,6 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: frame.c,v 1.67 2004/11/17 14:26:06 menno Exp $
  */
 
 /*
@@ -42,7 +41,6 @@
 #include "aacquant.h"
 #include "util.h"
 #include "huffman.h"
-#include "psych.h"
 #include "tns.h"
 #include "ltp.h"
 #include "backpred.h"
@@ -66,12 +64,8 @@ static const psymodellist_t psymodellist[] = {
 
 static SR_INFO srInfo[12+1];
 
-// base bandwidth for q=100
-static const int bwbase = 16000;
-// bandwidth multiplier (for quantiser)
-static const int bwmult = 120;
-// max bandwidth/samplerate ratio
-static const double bwfac = 0.45;
+// default bandwidth/samplerate ratio
+static const double bwfac = 0.42;
 
 
 int FAACAPI faacEncGetVersion( char **faac_id_string,
@@ -87,8 +81,9 @@ int FAACAPI faacEncGetVersion( char **faac_id_string,
 }
 
 
-int FAACAPI faacEncGetDecoderSpecificInfo(faacEncHandle hEncoder,unsigned char** ppBuffer,unsigned long* pSizeOfDecoderSpecificInfo)
+int FAACAPI faacEncGetDecoderSpecificInfo(faacEncHandle hpEncoder,unsigned char** ppBuffer,unsigned long* pSizeOfDecoderSpecificInfo)
 {
+    faacEncStruct* hEncoder = (faacEncStruct*)hpEncoder;
     BitStream* pBitStream = NULL;
 
     if((hEncoder == NULL) || (ppBuffer == NULL) || (pSizeOfDecoderSpecificInfo == NULL)) {
@@ -118,16 +113,18 @@ int FAACAPI faacEncGetDecoderSpecificInfo(faacEncHandle hEncoder,unsigned char**
 }
 
 
-faacEncConfigurationPtr FAACAPI faacEncGetCurrentConfiguration(faacEncHandle hEncoder)
+faacEncConfigurationPtr FAACAPI faacEncGetCurrentConfiguration(faacEncHandle hpEncoder)
 {
+    faacEncStruct* hEncoder = (faacEncStruct*)hpEncoder;
     faacEncConfigurationPtr config = &(hEncoder->config);
 
     return config;
 }
 
-int FAACAPI faacEncSetConfiguration(faacEncHandle hEncoder,
+int FAACAPI faacEncSetConfiguration(faacEncHandle hpEncoder,
                                     faacEncConfigurationPtr config)
 {
+    faacEncStruct* hEncoder = (faacEncStruct*)hpEncoder;
 	int i;
 
     hEncoder->config.allowMidside = config->allowMidside;
@@ -174,7 +171,7 @@ int FAACAPI faacEncSetConfiguration(faacEncHandle hEncoder,
 #endif
 
     if (config->bitRate && !config->bandWidth)
-    {	
+    {
 		static struct {
 			int rate; // per channel at 44100 sampling frequency
 			int cutoff;
@@ -196,6 +193,7 @@ int FAACAPI faacEncSetConfiguration(faacEncHandle hEncoder,
 			{47000, 10000},
 			{64000, 16000},
 			{76000, 20000},
+			{128000, 20000},
 #endif
 			{0, 0}
 		};
@@ -237,20 +235,15 @@ int FAACAPI faacEncSetConfiguration(faacEncHandle hEncoder,
 			config->bandWidth = f1;
 
 #ifndef DRM
-		config->bandWidth =
-				(double)config->bandWidth * hEncoder->sampleRate / 44100;
-		config->bitRate = tmpbitRate * hEncoder->sampleRate / 44100;
+        config->bandWidth = config->bitRate * hEncoder->sampleRate * bwfac / 60000;
 #endif
-
-		if (config->bandWidth > bwbase)
-		  config->bandWidth = bwbase;
-	}
+    }
 
     hEncoder->config.bitRate = config->bitRate;
 
     if (!config->bandWidth)
     {
-        config->bandWidth = (config->quantqual - 100) * bwmult + bwbase;
+        config->bandWidth = bwfac * hEncoder->sampleRate;
     }
 
     hEncoder->config.bandWidth = config->bandWidth;
@@ -277,14 +270,14 @@ int FAACAPI faacEncSetConfiguration(faacEncHandle hEncoder,
 		config->psymodelidx = (sizeof(psymodellist) / sizeof(psymodellist[0])) - 2;
 
     hEncoder->config.psymodelidx = config->psymodelidx;
-    hEncoder->psymodel = psymodellist[hEncoder->config.psymodelidx].model;
+    hEncoder->psymodel = (psymodel_t *)psymodellist[hEncoder->config.psymodelidx].ptr;
     hEncoder->psymodel->PsyInit(&hEncoder->gpsyInfo, hEncoder->psyInfo, hEncoder->numChannels,
 			hEncoder->sampleRate, hEncoder->srInfo->cb_width_long,
 			hEncoder->srInfo->num_cb_long, hEncoder->srInfo->cb_width_short,
 			hEncoder->srInfo->num_cb_short);
 	
 	/* load channel_map */
-	for( i = 0; i < 64; i++ )
+	for( i = 0; i < MAX_CHANNELS; i++ )
 		hEncoder->config.channel_map[i] = config->channel_map[i];
 
     /* OK */
@@ -297,7 +290,10 @@ faacEncHandle FAACAPI faacEncOpen(unsigned long sampleRate,
                                   unsigned long *maxOutputBytes)
 {
     unsigned int channel;
-    faacEncHandle hEncoder;
+    faacEncStruct* hEncoder;
+
+    if (numChannels > MAX_CHANNELS)
+	return NULL;
 
     *inputSamples = FRAME_LEN*numChannels;
     *maxOutputBytes = (6144/8)*numChannels;
@@ -328,17 +324,15 @@ faacEncHandle FAACAPI faacEncOpen(unsigned long sampleRate,
     hEncoder->config.useTns = 0;
     hEncoder->config.bitRate = 0; /* default bitrate / channel */
     hEncoder->config.bandWidth = bwfac * hEncoder->sampleRate;
-    if (hEncoder->config.bandWidth > bwbase)
-		hEncoder->config.bandWidth = bwbase;
     hEncoder->config.quantqual = 100;
     hEncoder->config.psymodellist = (psymodellist_t *)psymodellist;
     hEncoder->config.psymodelidx = 0;
     hEncoder->psymodel =
-      hEncoder->config.psymodellist[hEncoder->config.psymodelidx].model;
+      (psymodel_t *)hEncoder->config.psymodellist[hEncoder->config.psymodelidx].ptr;
     hEncoder->config.shortctl = SHORTCTL_NORMAL;
 
 	/* default channel map is straight-through */
-	for( channel = 0; channel < 64; channel++ )
+	for( channel = 0; channel < MAX_CHANNELS; channel++ )
 		hEncoder->config.channel_map[channel] = channel;
 	
     /*
@@ -401,8 +395,9 @@ faacEncHandle FAACAPI faacEncOpen(unsigned long sampleRate,
     return hEncoder;
 }
 
-int FAACAPI faacEncClose(faacEncHandle hEncoder)
+int FAACAPI faacEncClose(faacEncHandle hpEncoder)
 {
+    faacEncStruct* hEncoder = (faacEncStruct*)hpEncoder;
     unsigned int channel;
 
     /* Deinitialize coder functions */
@@ -441,13 +436,14 @@ int FAACAPI faacEncClose(faacEncHandle hEncoder)
     return 0;
 }
 
-int FAACAPI faacEncEncode(faacEncHandle hEncoder,
+int FAACAPI faacEncEncode(faacEncHandle hpEncoder,
                           int32_t *inputBuffer,
                           unsigned int samplesInput,
                           unsigned char *outputBuffer,
                           unsigned int bufferSize
                           )
 {
+    faacEncStruct* hEncoder = (faacEncStruct*)hpEncoder;
     unsigned int channel, i;
     int sb, frameBytes;
     unsigned int offset;
@@ -733,8 +729,6 @@ int FAACAPI faacEncEncode(faacEncHandle hEncoder,
     for (channel = 0; channel < numChannels; channel++) {
 		if (coderInfo[channel].block_type == ONLY_SHORT_WINDOW) {
 			SortForGrouping(&coderInfo[channel],
-					&hEncoder->psyInfo[channel],
-					&channelInfo[channel],
 					hEncoder->srInfo->cb_width_short,
 					hEncoder->freqBuff[channel]);
 		}
@@ -763,15 +757,15 @@ int FAACAPI faacEncEncode(faacEncHandle hEncoder,
     /* Quantize and code the signal */
     for (channel = 0; channel < numChannels; channel++) {
         if (coderInfo[channel].block_type == ONLY_SHORT_WINDOW) {
-            AACQuantize(&coderInfo[channel], &hEncoder->psyInfo[channel],
-					&channelInfo[channel], hEncoder->srInfo->cb_width_short,
-					hEncoder->srInfo->num_cb_short, hEncoder->freqBuff[channel],
-					&(hEncoder->aacquantCfg));
+            AACQuantize(&coderInfo[channel],
+                        hEncoder->srInfo->cb_width_short,
+                        hEncoder->srInfo->num_cb_short, hEncoder->freqBuff[channel],
+                        &(hEncoder->aacquantCfg));
         } else {
-            AACQuantize(&coderInfo[channel], &hEncoder->psyInfo[channel],
-					&channelInfo[channel], hEncoder->srInfo->cb_width_long,
-					hEncoder->srInfo->num_cb_long, hEncoder->freqBuff[channel],
-					&(hEncoder->aacquantCfg));
+            AACQuantize(&coderInfo[channel],
+                        hEncoder->srInfo->cb_width_long,
+                        hEncoder->srInfo->num_cb_long, hEncoder->freqBuff[channel],
+                        &(hEncoder->aacquantCfg));
         }
     }
 
@@ -1113,140 +1107,3 @@ static SR_INFO srInfo[12+1] =
     { -1 }
 };
 #endif
-
-/*
-$Log: frame.c,v $
-Revision 1.67  2004/11/17 14:26:06  menno
-Infinite loop fix
-dunno if this is good, encoder might be tuned to use energies from before MS encoding. But since the MS encoded samples are used in quantisation this might actually be better. Please test.
-
-Revision 1.66  2004/11/04 12:51:09  aforanna
-version number updated to 1.24.1 due to changes in Winamp and CoolEdit plugins
-
-Revision 1.65  2004/07/18 09:34:24  corrados
-New bandwidth settings for DRM, improved quantization quality adaptation (almost constant bit-rate now)
-
-Revision 1.64  2004/07/13 17:56:37  corrados
-bug fix with new object type definitions
-
-Revision 1.63  2004/07/08 14:01:25  corrados
-New scalefactorband table for 960 transform length, bug fix in HCR
-
-Revision 1.62  2004/07/04 12:10:52  corrados
-made faac compliant with Digital Radio Mondiale (DRM) (DRM macro must be set)
-implemented HCR tool, VCB11, CRC, scalable bitstream order
-note: VCB11 only uses codebook 11! TODO: implement codebooks 16-32
-960 transform length is not yet implemented (TODO)! Use 1024 for encoding and 960 for decoding, resulting in a lot of artefacts
-
-Revision 1.61  2004/05/03 11:37:16  danchr
-bump version to unstable 1.24+
-
-Revision 1.60  2004/04/13 13:47:33  danchr
-clarify release <> unstable status
-
-Revision 1.59  2004/04/02 14:56:17  danchr
-fix name clash w/ libavcodec: fft_init -> fft_initialize
-bump version number to 1.24 beta
-
-Revision 1.58  2004/03/17 13:34:20  danchr
-Automatic, untuned setting of lowpass for VBR.
-
-Revision 1.57  2004/03/15 20:16:42  knik
-fixed copyright notice
-
-Revision 1.56  2004/01/23 10:22:26  stux
-*** empty log message ***
-
-Revision 1.55  2003/12/17 20:59:55  knik
-changed default cutoff to 16k
-
-Revision 1.54  2003/11/24 18:09:12  knik
-A safe version of faacEncGetVersion() without string length problem.
-Removed Stux from copyright notice. I don't think he contributed something very
-substantial to faac and this is not the right place to list all contributors.
-
-Revision 1.53  2003/11/16 05:02:52  stux
-moved global tables from fft.c into hEncoder FFT_Tables. Add fft_init and fft_terminate, flowed through all necessary changes. This should remove at least one instance of a memory leak, and fix some thread-safety problems. Version update to 1.23.3
-
-Revision 1.52  2003/11/15 08:13:42  stux
-added FaacEncGetVersion(), version 1.23.2, added myself to faacCopyright :-P, does vanity know no bound ;)
-
-Revision 1.51  2003/11/10 17:48:00  knik
-Allowed independent bitRate and bandWidth setting.
-Small fixes.
-
-Revision 1.50  2003/10/29 10:31:25  stux
-Added channel_map to FaacEncHandle, facilitates free generalised channel remapping in the faac core. Default is straight-through, should be *zero* performance hit... and even probably an immeasurable performance gain, updated FAAC_CFG_VERSION to 104 and FAAC_VERSION to 1.22.0
-
-Revision 1.49  2003/10/12 16:43:39  knik
-average bitrate control made more stable
-
-Revision 1.48  2003/10/12 14:29:53  knik
-more accurate average bitrate control
-
-Revision 1.47  2003/09/24 16:26:54  knik
-faacEncStruct: quantizer specific data enclosed in AACQuantCfg structure.
-Added config option to enforce block type.
-
-Revision 1.46  2003/09/07 16:48:31  knik
-Updated psymodel call. Updated bitrate/cutoff mapping table.
-
-Revision 1.45  2003/08/23 15:02:13  knik
-last frame moved back to the library
-
-Revision 1.44  2003/08/15 11:42:08  knik
-removed single silent flush frame
-
-Revision 1.43  2003/08/11 09:43:47  menno
-thread safety, some tables added to the encoder context
-
-Revision 1.42  2003/08/09 11:39:30  knik
-LFE support enabled by default
-
-Revision 1.41  2003/08/08 10:02:09  menno
-Small fix
-
-Revision 1.40  2003/08/07 08:17:00  knik
-Better LFE support (reduced bandwidth)
-
-Revision 1.39  2003/08/02 11:32:10  stux
-added config.inputFormat, and associated defines and code, faac now handles native endian 16bit, 24bit and float input. Added faacEncGetDecoderSpecificInfo to the dll exports, needed for MP4. Updated DLL .dsp to compile without error. Updated CFG_VERSION to 102. Version number might need to be updated as the API has technically changed. Did not update libfaac.pdf
-
-Revision 1.38  2003/07/10 19:17:01  knik
-24-bit input
-
-Revision 1.37  2003/06/26 19:20:09  knik
-Mid/Side support.
-Copyright info moved from frontend.
-Fixed memory leak.
-
-Revision 1.36  2003/05/12 17:53:16  knik
-updated ABR table
-
-Revision 1.35  2003/05/10 09:39:55  knik
-added approximate ABR setting
-modified default cutoff
-
-Revision 1.34  2003/05/01 09:31:39  knik
-removed ISO psyodel
-disabled m/s coding
-fixed default bandwidth
-reduced max_sfb check
-
-Revision 1.33  2003/04/13 08:37:23  knik
-version number moved to version.h
-
-Revision 1.32  2003/03/27 17:08:23  knik
-added quantizer quality and bandwidth setting
-
-Revision 1.31  2002/10/11 18:00:15  menno
-small bugfix
-
-Revision 1.30  2002/10/08 18:53:01  menno
-Fixed some memory leakage
-
-Revision 1.29  2002/08/19 16:34:43  knik
-added one additional flush frame
-fixed sample buffer memory allocation
-
-*/
